@@ -211,6 +211,9 @@ int main(int argc, char **argv)
 
     envquery(argc, argv);
 #ifdef ENABLE_NLS
+#if defined(_CLANGD) && !defined(LOCALEDIR)
+#define LOCALEDIR "po/"
+#endif
     setlocale (LC_ALL, "");
     bindtextdomain(PACKAGE, LOCALEDIR);
     textdomain(PACKAGE);
@@ -668,6 +671,7 @@ int main(int argc, char **argv)
 		snprintf(tmpbuf, pplen, password_prompt,
 			ctl->remotename, ctl->server.pollname);
 		ctl->password = xstrdup((char *)fm_getpassword(tmpbuf));
+		fm_clearpassword();
 		free(tmpbuf);
 	    }
 	}
@@ -1018,6 +1022,7 @@ static void optmerge(struct query *h2, struct query *h1, int force)
 
 #define   FLAG_MERGE(fld) do { if (force ? !!h1->fld : !h2->fld) h2->fld = h1->fld; } while (0)
 #define STRING_MERGE(fld) do { if (force ? !!h1->fld : !h2->fld) { if (h2->fld) free((void *)h2->fld), h2->fld = 0; if (h1->fld) { if (h1->fld != STRING_DISABLED) h2->fld = xstrdup(h1->fld); else h2->fld = STRING_DISABLED; } } } while (0)
+#define STRING_MERGE_SAFE(fld) do { if (force ? !!h1->fld : !h2->fld) { if (h2->fld) { fm_safe_clearcstr((void *)h2->fld); free((void *)h2->fld); h2->fld = 0; } if (h1->fld) { if (h1->fld != STRING_DISABLED) h2->fld = xstrdup(h1->fld); else h2->fld = STRING_DISABLED; } } } while (0)
     STRING_MERGE(server.via);
     FLAG_MERGE(server.protocol);
     STRING_MERGE(server.service);
@@ -1034,7 +1039,7 @@ static void optmerge(struct query *h2, struct query *h1, int force)
     FLAG_MERGE(server.uidl);
     STRING_MERGE(server.principal);
     STRING_MERGE(server.esmtp_name);
-    STRING_MERGE(server.esmtp_password);
+    STRING_MERGE_SAFE(server.esmtp_password);
 
 #ifdef CAN_MONITOR
     STRING_MERGE(server.interface);
@@ -1049,7 +1054,7 @@ static void optmerge(struct query *h2, struct query *h1, int force)
 
     FLAG_MERGE(wildcard);
     STRING_MERGE(remotename);
-    STRING_MERGE(password);
+    STRING_MERGE_SAFE(password);
     STRING_MERGE(mda);
     STRING_MERGE(bsmtp);
     FLAG_MERGE(listener);
@@ -1537,10 +1542,13 @@ static void terminate_poll(int sig)
 }
 
 static void terminate_run(int sig)
-/* to be executed on normal or signal-induced termination */
+/* to be executed on normal or signal-induced termination
+ * - will not return if sig != 0, but log the signal and exit. */
 {
     struct query	*ctl;
 
+    clear_free_visbuf();
+
     terminate_poll(sig);
 
     /* 
@@ -1556,12 +1564,17 @@ static void terminate_run(int sig)
      */
     for (ctl = querylist; ctl; ctl = ctl->next)
 	if (ctl->password)
-	  memset(ctl->password, '\0', strlen(ctl->password));
+	  // revised on 2025-09-09 for fetchmail 6.5.5 release
+	  // compilers are free to optimize away a memset(),
+	  // se we need to use something else.
+	  fm_safe_clearcstr(ctl->password);
 
-    if (activecount == 0)
-	exit(PS_NOMAIL);
-    else
-	exit(successes ? PS_SUCCESS : querystatus);
+    if (sig) {
+	if (activecount == 0)
+	    exit(PS_NOMAIL);
+	else
+	    exit(successes ? PS_SUCCESS : querystatus);
+    }
 }
 
 /*
