@@ -5,9 +5,15 @@
  * For license terms, see the file COPYING in this directory.
  */
 
-#include "config.h"
+#if defined(_CLANGD) || defined(__cplusplus)
+#define restrict  /* NOTHING */ // NOSONAR: we intentionally hide this for C++
+#endif
 
-struct addrinfo;
+#if defined(_CLANGD)
+#define LOCALEDIR "/nonexist"
+#endif
+
+#include "config.h"
 
 /* We need this for size_t */
 #include <sys/types.h>
@@ -18,6 +24,7 @@ struct addrinfo;
 
 #include <sys/socket.h>
 #include <netdb.h>
+#include <signal.h> // we may need SIGKILL
 #include <stdio.h>
 #include <stdbool.h>
 
@@ -656,9 +663,26 @@ void deal_with_sigchld(void);
 void null_signal_handler(int sig);
 SIGHANDLERTYPE set_signal_handler(int sig, SIGHANDLERTYPE handler);
 int daemonize(const char *);
-char *fm_getpassword(char *);
+
+/* getpass.c */
+/** fm_getpassword has two modes
+ * - if isatty(0) is true, displays prompt on stderr and reads password from
+ *   /dev/tty (the process's controlling terminal), and prints a blank.
+ *   ECHOing input to the terminal is suppressed through the termios interface.
+ * - if isatty(0) is false, or /dev/tty cannot be opened, just read input from
+ *   stdin until EOF is encountered.
+ * Returns NULL on error, a pointer to a static buffer containing a possibly
+ * truncated C string otherwise.
+ */
+char *fm_getpassword(const char *prompt);
+/** fm_getpassword clears the internal static password buffer safely. */
+void fm_clearpassword(void);
+
+struct addrinfo; // forward declaration JIC
+
 void escapes(const char *, char *);
-char *visbuf(const char *);
+char *visbuf(const char *str); /**< prints str escaped. Returns pointer to malloc()d buffer, which can be released with zap_free_visbuf */
+void clear_free_visbuf(void); /**< clears then releases static buffer used by visbuf */
 const char *showproto(int);
 void dump_config(struct runctl *runp, struct query *querylist);
 int is_host_alias(const char *, struct query *, struct addrinfo **);
@@ -685,6 +709,20 @@ char *rfc2047e(const char*, const char *);
 char *stpcpy(char *, const char*);
 #endif
 
+#ifndef HAVE_EXPLICIT_BZERO
+void explicit_bzero(void *buffer, size_t size);
+#endif
+
+#ifdef HAVE_MEMSET_EXPLICIT
+// this is just a shortcut, our own explicit_bzero should try memset_explicit 
+// first, too
+# define fm_safe_clearmem(ptr, size) (memset_explicit((ptr), 0, (size)))
+#else
+// fall back to the systems, or to our own, explicit_bzero
+# define fm_safe_clearmem(ptr, size) (explicit_bzero((ptr), (size)))
+#endif
+#define fm_safe_clearcstr(cstr) (fm_safe_clearmem(cstr, strlen((const char *)cstr)))
+
 #define ROOT_UID 0
 
 extern int mailserver_socket_temp;
