@@ -130,7 +130,9 @@ static void SMTP_auth(int sock, char smtp_mode, char *username, char *password,
 			tmp[c] = '\0';
 
 		to64frombits(b64buf, tmp, len, sizeof b64buf);
-		SockPrintf(sock, "AUTH PLAIN %s\r\n", b64buf);
+		fm_safe_clearmem(tmp, sizeof(tmp));
+		SockPrintfClear(sock, "AUTH PLAIN %s\r\n", b64buf);
+		fm_safe_clearmem(b64buf, sizeof(b64buf));
 		SMTP_ok(sock, smtp_mode, TIMEOUT_DEFAULT);
 	}
 	else if (strstr(buf, "LOGIN")) {
@@ -167,7 +169,8 @@ static void SMTP_auth(int sock, char smtp_mode, char *username, char *password,
 			return;
 		}
 		to64frombits(b64buf, password, strlen(password), sizeof b64buf);
-		SockPrintf(sock, "%s\r\n", b64buf);
+		SockPrintfClear(sock, "%s\r\n", b64buf);
+		fm_safe_clearmem(b64buf, sizeof(b64buf));
 		SMTP_ok(sock, smtp_mode, TIMEOUT_DEFAULT);
 	}
 	return;
